#include <stdio.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>

#include "fail.h"
#include "fork.h"

static void replace(char *src, char *dest, char *pty) {
  int i = 0;
  int j = 0;
  while (src[i] != '\0') {
    if (src[i] == '%' && src[i+1] == 'p') {
      while (*pty != '\0') {
	dest[j++] = *pty;
	pty++;
      }
      i += 2;
      break;
    } else {
      dest[j++] = src[i++];
    }
  }
  while (src[i] != '\0') {
    dest[j++] = src[i++];
  }
  dest[j] = '\0';
}

static int commpid;

static void sigchld(int dummy) {
  int status;
  
  if (waitpid(commpid, &status, WNOHANG) > 0) {
    info("Comm program exited with status %d.\n", WEXITSTATUS(status));
    exit(WEXITSTATUS(status));
  }
}

void forkcomm(char *commprog, char *pty) {
  static char *args[4] = { "/bin/sh", "-c", "", NULL };

  args[2] = (char *) malloc(strlen(commprog) + strlen(pty));
  replace(commprog, args[2], pty);

  signal(SIGCHLD, sigchld);
  commpid = fork();

  switch (commpid) {
  case -1:
    fail("Unable to fork(): %s\n.", strerror(errno));
  case 0:
    signal(SIGCHLD, SIG_DFL);
    execv(args[0], args);
    fail("Unable to exec comm program: %s\n", strerror(errno));
  default:
    signal(SIGINT, SIG_IGN);
    signal(SIGQUIT, SIG_IGN);
    break;
  }
}
