#include <netinet/in.h>	/*sockaddr_in*/
#include <netdb.h>	/*hostent,(gethostbyname)*/
#include <string.h>	/*(memset)*/
#include <sys/socket.h>	/*AF_INET*/
#include <sys/types.h>	/*->socket.h (u_short etc.)*/

#include "sock.h"
#include "fail.h"

static struct {
  int fd;
  int alive;
} sock;

void sock_close() {
  if (sock.alive != 0) {
    shutdown(sock.fd, 2);
    sock.alive = 0;
  }
  if (sock.fd > 0) {
    close(sock.fd);
    sock.fd = 0;
  }
}

int sock_open(char *host, int port) {
  struct sockaddr_in sa;
  struct hostent *he;
  int tmp;

  memset(&sa, 0, sizeof(sa));

  sock.fd = 0;
  sock.alive = 0;

  he = gethostbyname(host);
  if (NULL == he) {
    warn("Host address lookup failed. ");
    return 0;
  }

  memcpy(&sa.sin_addr.s_addr, he->h_addr, he->h_length);
  sa.sin_port = htons(port);
  sa.sin_family = AF_INET;

  sock.fd = socket(AF_INET, SOCK_STREAM, 0);
  if (sock.fd < 0) {
    warn("Error creating socket. ");
    return 0;
  }

  tmp = 1;
  if (setsockopt(sock.fd, SOL_SOCKET, SO_OOBINLINE, &tmp, sizeof(tmp)) < 0) {
    warn("Error setting socket options. ");
    return 0;
  }

  if (connect(sock.fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
    sock_close();
    warn("Error connecting socket to desired host. ");
    return 0;
  }
  sock.alive = 1;
  return 1;
}

int sock_alive() {
  return sock.alive;
}

int sock_fd() {
  return sock.fd;
}
