#include <stdio.h>	/*stderr,(perror)*/
#include <unistd.h>	/*(read,write)*/
#include <sys/time.h>	/*->ttybuf.h (timeval)*/

#include "ttybuf.h"
#include "sockbuf.h"
#include "sock.h"
#include "fail.h"

#define TTYBUFR_SIZE	1024
#define SOCKBUFR_SIZE	1024
#define SOCKBUFW_SIZE (2 * TTYBUFR_SIZE) /* this seems to be any number */
#define SOCKBUFW_SIZE_A (SOCKBUFW_SIZE + TTYBUFR_SIZE) /* important */

static struct {
  unsigned char buf[SOCKBUFR_SIZE];
  unsigned char *ptr;
  unsigned char *end;
} sockr;

struct {
  unsigned char buf[SOCKBUFW_SIZE_A];
  unsigned char *top;
  unsigned char *ptr;
  int stop;
} sockw;

void sockr_reset() {
  sockr.ptr = sockr.end = sockr.buf;
}

void sockw_reset() {
  sockw.ptr = sockw.top = sockw.buf;
  sockw.stop = 0;
}

int sockw_has_data() {
  return sockw.ptr > sockw.buf;
}

int sockw_ready() {
  return !sockw.stop;
}

int sockr_has_data() {
  return sockr.ptr < sockr.end;
}

void sock_read() {
  int l;
  
  l = recv(sock_fd(), sockr.buf, sizeof(sockr.buf), 0);
  if (l <= 0) {
    sock_close();
    if (l == 0) {
      fail("Connection closed by peer.\n");
    } else {
      warn("Error receiving from socket.\n");
      return;
    }
  }
  sockr.ptr = sockr.buf;
  sockr.end = sockr.buf + l;
}

void sock_write() {
  int wl, l;

  wl = sockw.ptr - sockw.top;
  if (wl == 0) return;

  l = send(sock_fd(), sockw.top, wl, 0);
  if (l <= 0) {
    sock_close();
    fail("Connection closed by peer.\n");
  } else if (l < wl) {
    sockw.top += l;
    return;
  }
  sockw.ptr = sockw.top = sockw.buf;
  sockw.stop = 0;
  return;
}

void sock_flush() {
  sock_write();
}

int sock_get1() {
  return (sockr.ptr >= sockr.end) ? -1 : *sockr.ptr++;
}

int sock_put1(unsigned char c) {
  if (sockw.ptr >= sockw.buf + SOCKBUFW_SIZE) { /* limit */
    if (sockw.ptr >= sockw.buf + SOCKBUFW_SIZE_A) { /*actual limit*/
      warn("Write socket buffer overrun.\n");
      return 0;
    } else {
      sockw.stop = 1; /* flow control */
      return 0;
    }
  }
  *sockw.ptr++ = c;
  return 1;
}

void sock_put(unsigned char *cp, int n) {
  for (; n > 0; n--, cp++) {
    sock_put1(*cp);
  }
}

static int (*handler) (unsigned char);

void sock_readloop() {
  int c;
  if (!sockr_has_data()) {
    sock_read();
  }
  while ((c = sock_get1()) >= 0) {
    if (!(*handler)(c)) {
      return;
    }
  }
}

void sock_sethandler(int (*h) (unsigned char)) {
  handler = h;
}
