#include <stdio.h>	/*stderr,(perror)*/
#include <unistd.h>	/*(read,write)*/
#include <sys/time.h>	/*->ttybuf.h (timeval)*/

#include "sockbuf.h"
#include "ttybuf.h"
#include "fail.h"

#define TTYBUFR_SIZE	1024
#define SOCKBUFR_SIZE	1024
#define TTYBUFW_SIZE (2 * SOCKBUFR_SIZE) /* this seems to be any number */
#define TTYBUFW_SIZE_A (TTYBUFW_SIZE + SOCKBUFR_SIZE) /* important */

static struct {
  unsigned char buf[TTYBUFR_SIZE];
  unsigned char *ptr;
  unsigned char *end;
} ttyr;

static struct {
  unsigned char buf[TTYBUFW_SIZE_A];
  unsigned char *top;
  unsigned char *ptr;
  int stop;
} ttyw;

static struct {
  int rfd;
  int wfd;
} tty;

void tty_set(int rfd, int wfd) {
  tty.rfd = rfd;
  tty.wfd = wfd;
}

int tty_rfd() {
  return tty.rfd;
}

int tty_wfd() {
  return tty.wfd;
}

void ttyr_reset() {
  ttyr.ptr = ttyr.end = ttyr.buf;
}

void ttyw_reset() {
  ttyw.ptr = ttyw.top = ttyw.buf;
  ttyw.stop = 0;
}

int ttyw_has_data() {
  return ttyw.ptr > ttyw.buf;
}

int ttyw_ready() {
  return !ttyw.stop;
}

int ttyr_has_data() {
  return ttyr.ptr < ttyr.end;
}

int tty_get1() {
  return (ttyr.ptr >= ttyr.end) ? -1 : *ttyr.ptr++;
}

void tty_read() {
  int l;

  l = read(tty.rfd, ttyr.buf, sizeof(ttyr.buf));
  if (l <= 0) {
    /* close socket here! */
    fail("Read pty closed.\n");
  }

  ttyr.ptr = ttyr.buf;
  ttyr.end = ttyr.buf + l;
}

void tty_write() {
  int wl, l;

  wl = ttyw.ptr - ttyw.top;
  if (wl == 0) return;

  l = write(tty.wfd, ttyw.top, wl);
  if (l <= 0) {
    /* close socket here! */
    fail("Write pty closed.\n");
  } else if (l < wl) {
    ttyw.top += l;
    return;
  }
  ttyw.ptr = ttyw.top = ttyw.buf;
  ttyw.stop = 0;
  return;
}

void tty_flush() {
  tty_write();
}

int tty_put1(unsigned char c) {
  if (ttyw.ptr >= ttyw.buf + TTYBUFW_SIZE) {
    if (ttyw.ptr >= ttyw.buf + TTYBUFW_SIZE_A) {
      warn("ttyw overrun.\n");
      return 0;
    } else {
      ttyw.stop = 1;
    }
  }
  *ttyw.ptr++ = c;
  return 1;
}

void tty_put(unsigned char *cp, int n) {
  for (; n > 0; n--, cp++) {
    tty_put1(*cp);
  }
}

static int (*handler) (unsigned char);

void tty_readloop() {
  int c;
  if (!ttyr_has_data()) {
    tty_read();
  }
  while ((c = tty_get1()) >= 0) {
    if (!(*handler)(c)) {
      return;
    }
  }
}

void tty_sethandler(int (*h) (unsigned char)) {
  handler = h;
}
